package org.sadigit.boundary.v1.endpoint;

import java.util.List;
import java.util.Map;

import org.sadigit.control.exception.CustomException;
import org.sadigit.model.request.CreateGangguanRequest;
import org.sadigit.model.request.CreateGangguanV2Request;
import org.sadigit.model.request.CreateKeluhanRequest;
import org.sadigit.model.request.CreateKeluhanV2Request;
import org.sadigit.model.request.GetCancelRequest;
import org.sadigit.model.request.GetCttRequest;
import org.sadigit.model.request.GetDataGarduV2Request;
import org.sadigit.model.request.GetDataRequest;
import org.sadigit.model.request.GetRatingPlgnRequest;
import org.sadigit.model.response.CreateGangguanResponse;
import org.sadigit.model.response.DetailPengaduanFlowResponse;
import org.sadigit.model.response.DetailPengaduanResponse;
import org.sadigit.model.response.GarduNyalaResponse;
import org.sadigit.model.response.GetCttResponse;
import org.sadigit.model.response.GarduPadamResponse;
import org.sadigit.model.response.GarduRencanaResponse;
import org.sadigit.model.response.GetFotoResponse;
import org.sadigit.model.response.GetHistoByIdpelResponse;
import org.sadigit.model.response.GetHistoriGangguanByIdResponse;
import org.sadigit.model.response.GetMasterJenisPengaduanResponse;
import org.sadigit.model.response.GetMasterJenisPengaduanResponseV2;
import org.sadigit.model.response.GetRatingPlnResponse;
import org.sadigit.model.response.base.ResponseModel;
import org.sadigit.model.response.base.ResponseModelGardu;
import org.sadigit.model.response.base.ResponseModelGarduV2;
import org.sadigit.model.response.base.ResponseModelOutReturn;
import org.sadigit.model.response.base.ResponseModelWithFlow;
import org.sadigit.service.api.v1.PlnMobileService;

import jakarta.inject.Inject;
import jakarta.ws.rs.BeanParam;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;

@Path("/api/apkt/plnmobile")
public class PlnMobileResource {

    @Inject
    PlnMobileService plnMobileService;

    @POST
    @Path(value = "/histogangguan")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<GetHistoriGangguanByIdResponse> histogangguan(@QueryParam("in_data") String in_data) {

        if (in_data == null || in_data.isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "in_data tidak boleh kosong"))
                    .build();
            throw new CustomException(null, error);
        }

        return plnMobileService.getHistoriGangguanById(in_data);
    }

    @POST
    @Path(value = "/detailpengaduan")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModelWithFlow<DetailPengaduanResponse, DetailPengaduanFlowResponse> detailpengaduan(@QueryParam("in_data") String in_data) {
        if (in_data == null || in_data.isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "in_data tidak boleh kosong"))
                    .build();
            throw new CustomException(null, error);
        }

        return plnMobileService.getDetailPengaduan(in_data);
    }

    @POST
    @Path(value = "/masterjp")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<List<GetMasterJenisPengaduanResponse>> masterjp() {
        return plnMobileService.getMasterJenisPengaduan();
    }

    @POST
    @Path(value = "/creategangguan")
    @Produces(MediaType.APPLICATION_JSON)
    public CreateGangguanResponse creategangguan(@BeanParam CreateGangguanRequest params) {
        params.checkParameter();
        return plnMobileService.createGangguan(params);
    }

    @POST
    @Path(value = "/creategangguanv2")
    @Produces(MediaType.APPLICATION_JSON)
    public CreateGangguanResponse creategangguanv2(@BeanParam CreateGangguanV2Request params) {
        params.checkParameter();
        return plnMobileService.createGangguanV2(params);
    }

    @POST
    @Path(value = "/createkeluhan")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<Map<String, String>> createkeluhan(@BeanParam CreateKeluhanRequest params) {
        params.checkParameter();
        return plnMobileService.createKeluhan(params);
    }

    @POST
    @Path(value = "/get_cancel")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModelOutReturn<String> getcancel(@BeanParam GetCancelRequest params) {
        params.checkParameter();
        return plnMobileService.batalkanPengaduan(params);
    }

    @POST
    @Path(value = "/gethistobyidpel")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<List<GetHistoByIdpelResponse>> gethistobyidpel(
            @QueryParam("in_data") String in_data) {
        if (in_data == null || in_data.isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "in_data tidak boleh kosong"))
                    .build();
            throw new CustomException(null, error);
        }

        return plnMobileService.getHistoriGangguanByIdpel(in_data);
    }

    @POST
    @Path(value = "/rating")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<List<GetRatingPlnResponse>> rating(@BeanParam GetRatingPlgnRequest request) {
        request.checkParameter();
        return plnMobileService.getRating(request);
    }

    @POST
    @Path(value = "/getdatagardu")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModelGardu<GarduPadamResponse, GarduNyalaResponse, GarduPadamResponse, GarduRencanaResponse> getdatagardu() {
        return plnMobileService.getDataGardu();
    }

    @POST
    @Path(value = "/getdatagarduv2")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModelGarduV2<Object> getdatagarduv2(@BeanParam GetDataGarduV2Request params) {
        params.checkParameter();
        return plnMobileService.getDataGarduV2(params);
    }

    @POST
    @Path(value = "/masterjpv2")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<List<GetMasterJenisPengaduanResponseV2>> masterjpv2() {
        return plnMobileService.getMasterJenisPengaduanV2();
    }

    @POST
    @Path(value = "/createkeluhanV2")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<Map<String, String>> createkeluhanV2(CreateKeluhanV2Request params) {
        params.checkParameter();
        return plnMobileService.createKeluhanV2(params);
    }

    @POST
    @Path(value = "/GET_FOTO")
    @Produces(MediaType.APPLICATION_JSON)
    public GetFotoResponse<List<Map<String, String>>> getFoto(@QueryParam("p_reportnumber") String pReportNumber) {

        if (pReportNumber == null || pReportNumber.isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "No laporan tidak boleh kosong"))
                    .build();
            throw new CustomException(null, error);
        }
        return plnMobileService.getFoto(pReportNumber);
    }

    @POST
    @Path(value = "/GET_CTT")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<List<GetCttResponse>> GET_CTT(@BeanParam GetCttRequest params) {
        params.checkParameter();
        return plnMobileService.getCtt(params);
    }

    @POST
    @Path(value = "/get_data")
    @Produces(MediaType.APPLICATION_JSON)
    public ResponseModel<Object> get_data(@BeanParam GetDataRequest params) {
        return plnMobileService.getData(params);       
    }
}
