package org.sadigit.entity.transaksi;

import java.time.LocalDateTime;
import java.util.List;

import org.hibernate.annotations.Immutable;
import org.locationtech.jts.geom.Geometry;
import org.sadigit.adapter.GeometryAdapter;
import org.sadigit.entity.referensi.Cuaca;
import org.sadigit.entity.referensi.Dampak;
import org.sadigit.entity.referensi.Equipment;
import org.sadigit.entity.referensi.Fasilitas;
import org.sadigit.entity.referensi.GrupPenyebabGangguan;
import org.sadigit.entity.referensi.Kelompok;
import org.sadigit.entity.referensi.KelompokUtama;
import org.sadigit.entity.referensi.PenyebabGangguan;
import org.sadigit.entity.referensi.Posko;
import org.sadigit.entity.referensi.Prioritas;
import org.sadigit.entity.referensi.SubFasilitas;
import org.sadigit.entity.referensi.SubTipePermasalahan;
import org.sadigit.entity.referensi.TarifDaya;
import org.sadigit.entity.referensi.Tindakan;
import org.sadigit.entity.referensi.TipePermasalahan;
import org.sadigit.entity.referensi.MasterKelurahan;
import org.sadigit.entity.referensi.MerkMeter;

import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import jakarta.json.bind.annotation.JsonbTypeAdapter;
import jakarta.persistence.*;
import lombok.Getter;
import lombok.Setter;

@Entity
@Setter
@Getter
@Table(name = "gangguan", schema = "apkt_transactional")
@Immutable
public class Gangguan extends PanacheEntityBase {

    @Id
    @SequenceGenerator(name = "gangguan_seq", sequenceName = "apkt_transactional.gangguan_seq", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "gangguan_seq")
    @Column(name = "id")
    private Long id;
    @Column(name = "id_pelanggan_no_meter")
    private Long id_pelanggan_no_meter;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_posko", foreignKey = @ForeignKey(name = "posko_fk"))
    private Posko master_posko;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_tipe_permasalahan", foreignKey = @ForeignKey(name = "id_tipe_permasalahan"))
    private TipePermasalahan tipePermasalahan;

    @Column(name = "alamat_pelanggan")
    private String alamat_pelanggan;

    @Column(name = "nama_pelapor")
    private String nama_pelapor;

    @Column(name = "telepon_pelapor")
    private String telepon_pelapor;

    @Column(name = "hp_pelapor")
    private String hp_pelapor;

    @Column(name = "fax_pelapor")
    private String fax_pelapor;

    @Column(name = "email_pelapor")
    private String email_pelapor;

    @Column(name = "alamat_pelapor")
    private String alamat_pelapor;

    @Column(name = "media")
    private String media;

    @Column(name = "waktu_padam")
    private LocalDateTime waktu_padam;

    @Column(name = "is_konfirmasi")
    private Boolean konfirmasi;

    @JsonbTypeAdapter(GeometryAdapter.class)
    @Column(name = "lokasi", columnDefinition = "geometry(Point,4326)")
    private Geometry lokasi;

    @Column(name = "no_laporan", unique = true)
    private String no_laporan;

    @Column(name = "keterangan")
    private String keterangan;

    @Column(name = "created_date")
    private LocalDateTime created_date;

    @Column(name = "created_by")
    private Long created_by;

    @Column(name = "updated_date")
    private LocalDateTime updated_date;

    @Column(name = "updated_by")
    private Long updated_by;

    @Column(name = "status_akhir")
    private String status_akhir;

    @Column(name = "tipe_wo")
    private Long tipe_wo;

    @Column(name = "unit")
    private Long unit;

    @Column(name = "eskalasi")
    private Long eskalasi;

    @Column(name = "permasalahan")
    private String permasalahan;

    @Column(name = "waktu_lapor")
    private LocalDateTime waktu_lapor;

    // @Column(name = "id_regu")
    // private Long id_regu;

    @Column(name = "waktu_penugasan")
    private LocalDateTime waktu_penugasan;

    @Column(name = "waktu_perjalanan")
    private LocalDateTime waktu_perjalanan;

    @Column(name = "waktu_pengerjaan")
    private LocalDateTime waktu_pengerjaan;

    @Column(name = "waktu_nyala_sementara")
    private LocalDateTime waktu_nyala_sementara;

    @Column(name = "waktu_nyala")
    private LocalDateTime waktu_nyala;

    @Column(name = "waktu_batal")
    private LocalDateTime waktu_batal;

    @Column(name = "waktu_selesai")
    private LocalDateTime waktu_selesai;

    @Column(name = "alasan")
    private String alasan;

    @Column(name = "patokan")
    private String patokan;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_sub_tipe_permasalahan", foreignKey = @ForeignKey(name = "gangguan_fk"))
    private SubTipePermasalahan sub_tipe_permasalahan;

    @Column(name = "tipe_gangguan")
    private Long tipe_gangguan;

    @Column(name = "alasan_pembatalan")
    private String alasan_pembatalan;

    @Column(name = "jarak")
    private Long jarak;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_kelompok", foreignKey = @ForeignKey(name = "kelompok"))
    private Kelompok kelompok;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_sub_fasilitas", foreignKey = @ForeignKey(name = "sub_fasilitas_fk"))
    private SubFasilitas sub_fasilitas;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_equipment", foreignKey = @ForeignKey(name = "equipment_fk"))
    private Equipment equipment;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_prioritas", foreignKey = @ForeignKey(name = "prioritas"))
    private Prioritas prioritas;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_penyebab", foreignKey = @ForeignKey(name = "penyebab"))
    private PenyebabGangguan penyebab;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_regu", referencedColumnName = "id")
    private MasterRegu regu;

    @Column(name = "jumlah_pelanggan_termarking")
    private Long jumlah_pelanggan_termarking;

    @Column(name = "jumlah_lapor")
    private Long jumlah_lapor;

    @Column(name = "waktu_konfirmasi")
    private LocalDateTime waktu_konfirmasi;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_fasilitas", foreignKey = @ForeignKey(name = "fasilitas"))
    private Fasilitas fasilitas;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_kelompok_utama", foreignKey = @ForeignKey(name = "kelompok_utama"))
    private KelompokUtama kelompok_utama;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_dampak", foreignKey = @ForeignKey(name = "dampak"))
    private Dampak dampak;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_grup_penyebab_gangguan", foreignKey = @ForeignKey(name = "grup_penyebab_gangguan"))
    private GrupPenyebabGangguan grup_penyebab_gangguan;

    @OneToMany(mappedBy = "gangguan", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<GangguanTransaksi> gangguan_transaksi;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_kelurahan", referencedColumnName = "id_kelurahan", foreignKey = @ForeignKey(name = "kelurahan"))
    private MasterKelurahan master_kelurahan;
    // |id_tindakan|id_cuaca|fax|keterangan_lokasi|waktu_fax|nama_media|waktu_media|merk_meter
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_tindakan", referencedColumnName = "id", foreignKey = @ForeignKey(name = "tindakan"))
    private Tindakan tindakan;
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_cuaca", referencedColumnName = "id", foreignKey = @ForeignKey(name = "cuaca"))
    private Cuaca cuaca;
    @Column(name = "fax")
    private String fax;
    @Column(name = "keterangan_lokasi")
    private String keterangan_lokasi;
    @Column(name = "waktu_fax")
    private LocalDateTime waktu_fax;
    @Column(name = "nama_media")
    private String nama_media;
    @Column(name = "waktu_media")
    private LocalDateTime waktu_media;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_merk_meter", referencedColumnName = "id", foreignKey = @ForeignKey(name = "merk_meter"))
    private MerkMeter master_merk_meter;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_tarif_daya", referencedColumnName = "id", foreignKey = @ForeignKey(name = "tarif_daya"))
    private TarifDaya tarifDaya;

    @Column(name = "alasan_nyala_sementara")
    private String alasan_nyala_sementara;
    @Column(name = "penyebab_sementara")
    private String penyebab_sementara;
    private String tindakan_telah_dilakukan_nyala_sementara;
    private String usulan_tindakan_selanjutnya;
    private String tindakan_telah_dilakukan_nyala;

    private String penyebab_nyala;
    private Long jumlah_pelanggan_padam;
    private String ctt;
}
