package org.sadigit.repository.entity.transaksi;

import java.util.List;

import org.sadigit.control.exception.CustomException;
import org.sadigit.entity.transaksi.GangguanTMTransaksi;
import io.quarkus.hibernate.orm.panache.PanacheRepository;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.transaction.Transactional;
import lombok.extern.slf4j.Slf4j;

@ApplicationScoped
@Slf4j
public class GangguanTMTransaksiRepository implements PanacheRepository<GangguanTMTransaksi> {

    public List<GangguanTMTransaksi> getGangguanTMNotFlagged(Integer limit) {
        String query = "(notif_flag = false or notif_flag is null) and jenis = 'Selesai' and cast(created_date as Date) = cast(CURRENT_DATE as Date)";

        try {
            return limit == null 
                ? this.find(query).list()
                : this.find(query).page(0, limit).list();
        } catch (Exception e) {
            log.info("Error: {}", e.getMessage());
            throw new CustomException(e.getMessage());
        }
        
    }

    @Transactional
    public GangguanTMTransaksi flagTransaksi(String noLaporan) {
        StringBuilder queryKe1 = new StringBuilder();
        queryKe1.append("waktu_read = CURRENT_DATE and ")
            .append("notif_flag = 1 where ")
            .append("cast(created_date as Date) < cast(CURRENT_DATE as date) and ")
            .append("waktu_read is null");

        this.update(queryKe1.toString());

        StringBuilder queryKe2 = new StringBuilder();
        queryKe2.append("waktu_read = CURRENT_DATE and ")
            .append("notif_flag = 1 where ")
            .append("gangguanTm.noGangguanTm = ?1 and ")
            .append("(flag = 0 or flag is null) and ")
            .append("(waktu_read) in ")
            .append("(select min(c.waktu_read) from GangguanTMTransaksi c where c.gangguanTm.noGangguanTm = ?1 and (c.notif_flag = 0 or c.notif_flag is null) and cast(c.created_date as Date) = cast(CURRENT_DATE as date))");

        this.update(queryKe2.toString(), noLaporan);

        return this.find("gangguanTm.noGangguanTm = ?1 and cast(created_date as Date) = cast(CURRENT_DATE as Date)", noLaporan).firstResult();
    }

    public List<GangguanTMTransaksi> getGangguanTMNotFlaggedNotSelesai(Integer limit) {

        StringBuilder sb = new StringBuilder();
        sb.append("jenis != 'Selesai' and ")
            .append("(notif_flag = false or notif_flag is null) and ")
            .append("cast(created_date as Date) = cast(CURRENT_DATE as Date) and ")
            .append("(gangguanTm.noGangguanTm, created_date) in ")
            .append("(select c.gangguanTm.noGangguanTm, min(c.created_date) from GangguanTMTransaksi c where c.jenis != 'Selesai' and (c.notif_flag = false or c.notif_flag is null) and cast(c.created_date as Date) = cast(current_date as Date) group by c.gangguanTm.noGangguanTm)");
        
        try {
            return limit == null
                ? this.find(sb.toString()).list()
                : this.find(sb.toString()).page(0, limit).list();
        } catch (Exception e) {
            log.info("Error: {}", e.getMessage());
            throw new CustomException(e.getMessage());
        }
        
    }
}
