package org.sadigit.repository.entity.transaksi;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.transaction.Transactional;

import org.locationtech.jts.geom.Point;
import org.locationtech.jts.io.ParseException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

import org.hibernate.HibernateException;
import org.sadigit.control.exception.CustomException;
import org.sadigit.model.request.CreateKeluhanRequest;
import org.sadigit.model.request.CreateKeluhanV2Request;
import org.sadigit.model.response.base.ResponseModel;
import org.sadigit.entity.masterYantek.Unit;
import org.sadigit.entity.referensi.MasterKelurahan;
import org.sadigit.entity.referensi.Media;
import org.sadigit.entity.referensi.SubTipeKeluhan;
import org.sadigit.entity.referensi.TipeKeluhan;
import org.sadigit.entity.referensi.Uid;
import org.sadigit.entity.transaksi.Keluhan;
import org.sadigit.entity.transaksi.KeluhanLaporUlang;
import org.sadigit.entity.transaksi.KeluhanTransaksi;
import org.sadigit.utils.GeomUtils;

import io.quarkus.hibernate.orm.panache.PanacheRepository;

@ApplicationScoped
public class KeluhanRepository implements PanacheRepository<Keluhan> {

    private String generateNoLaporan(Long id_uid) {
        // select unitcode from unit
        Unit unit = Unit.find("unitid", id_uid).firstResult();
        if (unit == null) {
            throw new CustomException("Unit tidak ditemukan");
        }
        String unitCode = unit.getUnitcode();
        String noLaporan = "";
        String prefix = "K";
        Keluhan keluhan = Keluhan.find("unit =?1 and date(created_date)=?2 order by created_date desc", id_uid, LocalDate.now())
                .firstResult();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyMMdd");
        if (keluhan != null) {
            String lastNoLaporan = keluhan.getNo_laporan();
            String currentDate = LocalDate.now().format(formatter);
            String lastNoLaporanNumber = lastNoLaporan.substring(lastNoLaporan.length() - 4);
            int lastNoLaporanNumberInt = Integer.parseInt(lastNoLaporanNumber);
            int newNoLaporanNumber = lastNoLaporanNumberInt + 1;
            String newNoLaporanNumberString = String.format("%05d", newNoLaporanNumber);
            noLaporan = prefix + unitCode + currentDate + newNoLaporanNumberString;
            System.out.println("Generated no laporan: " + noLaporan);

            return noLaporan;
        } else {
            LocalDate date = LocalDate.now(); // Get current date directly
            System.out.println("Parsed date: " + date);
            // Format the LocalDate object to a string using the formatter
            String formattedDate = date.format(formatter);
            System.out.println("Formatted date: " + formattedDate);
            noLaporan = prefix + unitCode + formattedDate + "0001";
            System.out.println("Generated no laporan: " + noLaporan);
            return noLaporan;
        }
    }

    @Transactional
    public ResponseModel<Map<String,String>> create(CreateKeluhanRequest prms) {

        if (prms.getIn_idpel() != null) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime earlier = now.minusHours(3);
            Keluhan keluhanExist = Keluhan.find("id_pelanggan_no_meter = ?1 and created_date between ?2 and ?3", prms.getIn_idpel(), earlier, now).firstResult();
            if (keluhanExist != null) {
                var error = ResponseModel.builder()
                        .data(Map.of("msg", "Pengaduan anda sedang di proses. Anda dapat melakukan pengaduan ulang, 3 jam dari pengaduan sebelumnya."))
                        .build();
                throw new CustomException(null, error);
            }
        }

        try {
            int sourceSrid = 4326;
            Keluhan keluhan = new Keluhan();
            KeluhanTransaksi keluhanTransaksi = new KeluhanTransaksi();
            Media media = Media.findById(10);
            String noLaporan = "";

            try {

                Point point = GeomUtils.wktToPoint(prms.getIn_long(), prms.getIn_lat(), sourceSrid);
                Uid uid = Uid.findById(prms.getIn_unitup());
                MasterKelurahan kelurahan = MasterKelurahan.find("nama_kelurahan", prms.getIn_kel()).firstResult();
                TipeKeluhan tipeKeluhan = TipeKeluhan.findById(prms.getIn_tipe_keluhan());
                SubTipeKeluhan subTipeKeluhan = SubTipeKeluhan.findById(prms.getIn_subtipe_keluhan());

                if (uid == null) {
                    throw new CustomException("Unit tidak ditemukan");
                }
                if (tipeKeluhan == null) {
                    throw new CustomException("Tipe Keluhan tidak ditemukan");
                }
                if (subTipeKeluhan == null) {
                    throw new CustomException("Sub Tipe Keluhan tidak ditemukan");
                }
                if (kelurahan == null) {
                    throw new CustomException("Kelurahan tidak ditemukan");
                }
                if (prms.getIn_idpel() != null) {
                    keluhan.setId_pelanggan_no_meter(Long.valueOf(prms.getIn_idpel()));
                }

                keluhan.setKelurahan(kelurahan);
                noLaporan = generateNoLaporan(Long.valueOf(prms.getIn_unitup()));
                keluhan.setNo_laporan(noLaporan);
                keluhan.setNama_pelapor(prms.getIn_nama());
                keluhan.setTelepon_pelapor(prms.getIn_hp());
                keluhan.setHp_pelapor(prms.getIn_hp());
                keluhan.setAlamat_pelapor(prms.getIn_alamat());
                keluhan.setMedia(media.getKeterangan());

                keluhan.setLokasi(point);
                keluhan.setKeterangan(prms.getIn_keterangan());
                keluhan.setCreated_date(LocalDateTime.now());
                keluhan.setCreated_by(99999L);
                keluhan.setStatus_akhir("Lapor");
                keluhan.setUnit(Long.valueOf(prms.getIn_unitup()));
                keluhan.setTipe_keluhan(tipeKeluhan.getIssuetypeid());
                keluhan.setTipe_sub_keluhan(subTipeKeluhan.getSubissuetypeid());

                keluhan.setPermasalahan(tipeKeluhan.getIssuetypename());
                keluhan.setCreated_date(LocalDateTime.now());
                keluhan.setKelurahan(kelurahan);
                keluhan.persist();

                keluhanTransaksi.setKeluhan(keluhan);
                keluhanTransaksi.setJenis("Lapor");
                keluhanTransaksi.setWaktu(LocalDateTime.now());
                keluhanTransaksi.setCreated_date(LocalDateTime.now());
                keluhanTransaksi.setCreated_by(99999L);
                KeluhanTransaksi.persist(keluhanTransaksi);

                if (!isPersistent(keluhan)) {
                    Keluhan.delete("no_laporan", keluhan.getNo_laporan());
                    throw new CustomException("keluhan tidak berhasil disimpan");
                }

            } catch (ParseException e) {
                System.err.println("Invalid WKT format: " + e.getMessage());
                throw new CustomException("Invalid WKT format: " + e.getMessage());
            }

            ResponseModel<Map<String,String>> responseModel = new ResponseModel<>();
            responseModel.setData(Map.of("reportnumber", noLaporan));
            return responseModel;
            
        } catch (HibernateException e) {
            throw new CustomException("Terjadi kesalahan dalam proses menyimpan data : " + e.getMessage());
        }
    }

    @Transactional
    public ResponseModel<Map<String,String>> createV2(CreateKeluhanV2Request prms) {

        if (prms.getIn_idpel() != null) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime earlier = now.minusHours(3);
            Keluhan keluhanExist = Keluhan.find("id_pelanggan_no_meter = ?1 and created_date between ?2 and ?3", prms.getIn_idpel(), earlier, now).firstResult();
            if (keluhanExist != null) {
                var error = ResponseModel.builder()
                        .data(Map.of("msg", "Pengaduan anda sedang di proses. Anda dapat melakukan pengaduan ulang, 3 jam dari pengaduan sebelumnya."))
                        .build();
                throw new CustomException(null, error);
            }
        }

        try {
            int sourceSrid = 4326;
            Keluhan keluhan = new Keluhan();
            KeluhanTransaksi keluhanTransaksi = new KeluhanTransaksi();
            Media media = Media.findById(10);
            String noLaporan = "";

            try {
                Point point = GeomUtils.wktToPoint(prms.getIn_long(), prms.getIn_lat(), sourceSrid);
                Uid uid = Uid.findById(prms.getIn_unitup());
                MasterKelurahan kelurahan = MasterKelurahan.find("nama_kelurahan", prms.getIn_kel()).firstResult();
                TipeKeluhan tipeKeluhan = TipeKeluhan.findById(prms.getIn_tipe_keluhan());
                SubTipeKeluhan subTipeKeluhan = SubTipeKeluhan.findById(prms.getIn_subtipe_keluhan());

                if (uid == null) {
                    throw new CustomException("Unit tidak ditemukan");
                }
                if (tipeKeluhan == null) {
                    throw new CustomException("Tipe Keluhan tidak ditemukan");
                }
                if (subTipeKeluhan == null) {
                    throw new CustomException("Sub Tipe Keluhan tidak ditemukan");
                }
                if (kelurahan == null) {
                    throw new CustomException("Kelurahan tidak ditemukan");
                }
                if (prms.getIn_idpel() != null) {
                    keluhan.setId_pelanggan_no_meter(Long.valueOf(prms.getIn_idpel()));
                }

                if (prms.getIn_laporanulang_ref() != null) {
                    Keluhan gangguanExist = Keluhan.find("no_laporan = ?1", prms.getIn_laporanulang_ref()).firstResult();
                    if (gangguanExist == null) {
                        var error = ResponseModel.builder()
                            .data(Map.of(
                                "msg", "Referensi no laporan tidak ditemukan",
                                "result", "0"
                            ))
                            .build();
                        throw new CustomException(null, error);
                    }
        
                    Long jumlahLaporUlang = KeluhanLaporUlang.find("no_laporan_referensi", prms.getIn_laporanulang_ref()).count();
                    KeluhanLaporUlang newKeluhanLaporUlang = new KeluhanLaporUlang();
                    newKeluhanLaporUlang.setLaporan_ke(jumlahLaporUlang);
                    newKeluhanLaporUlang.setNo_laporan_referensi(prms.getIn_laporanulang_ref());
                    newKeluhanLaporUlang.setNo_laporan_baru(noLaporan);
                    newKeluhanLaporUlang.setKeluhan(gangguanExist);
                    KeluhanLaporUlang.persist(newKeluhanLaporUlang);
                }

                keluhan.setKelurahan(kelurahan);
                noLaporan = generateNoLaporan(Long.valueOf(prms.getIn_unitup()));
                keluhan.setNo_laporan(noLaporan);
                keluhan.setNama_pelapor(prms.getIn_nama());
                keluhan.setTelepon_pelapor(prms.getIn_hp());
                keluhan.setHp_pelapor(prms.getIn_hp());
                keluhan.setAlamat_pelapor(prms.getIn_alamat());
                keluhan.setMedia(media.getKeterangan());

                keluhan.setLokasi(point);
                keluhan.setKeterangan(prms.getIn_keterangan());
                keluhan.setCreated_date(LocalDateTime.now());
                keluhan.setCreated_by(99999L);
                keluhan.setStatus_akhir("LAPOR");
                keluhan.setUnit(Long.valueOf(prms.getIn_unitup()));
                keluhan.setTipe_keluhan(tipeKeluhan.getIssuetypeid());
                keluhan.setTipe_sub_keluhan(subTipeKeluhan.getSubissuetypeid());

                keluhan.setPermasalahan(tipeKeluhan.getIssuetypename());
                keluhan.setCreated_date(LocalDateTime.now());
                keluhan.setKelurahan(kelurahan);
                keluhan.persist();

                keluhanTransaksi.setKeluhan(keluhan);
                keluhanTransaksi.setJenis("LAPOR");
                keluhanTransaksi.setWaktu(LocalDateTime.now());
                keluhanTransaksi.setCreated_date(LocalDateTime.now());
                keluhanTransaksi.setCreated_by(99999L);
                KeluhanTransaksi.persist(keluhanTransaksi);

                if (!isPersistent(keluhan)) {
                    Keluhan.delete("no_laporan", keluhan.getNo_laporan());
                    throw new CustomException("keluhan tidak berhasil disimpan");
                }

            } catch (ParseException e) {
                System.err.println("Invalid WKT format: " + e.getMessage());
                throw new CustomException("Invalid WKT format: " + e.getMessage());
            }

            ResponseModel<Map<String,String>> responseModel = new ResponseModel<>();
            responseModel.setData(Map.of("reportnumber", noLaporan));
            return responseModel;
            
        } catch (HibernateException e) {
            throw new CustomException("Terjadi kesalahan dalam proses menyimpan data : " + e.getMessage());
        }
    }

}
