package org.sadigit.service.api.v1;

import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.hibernate.HibernateException;
import org.sadigit.control.exception.CustomException;
import org.sadigit.control.exception.GetDataGarduV2Exception;
import org.sadigit.entity.referensi.Kecamatan;
import org.sadigit.entity.referensi.MasterKelurahan;
import org.sadigit.entity.referensi.Provinsi;
import org.sadigit.entity.referensi.RatingPln;
import org.sadigit.entity.referensi.SubTipePermasalahan;
import org.sadigit.entity.referensi.TipeKeluhan;
import org.sadigit.entity.saidisaifi.masterJaringan.MasterGarduIndukTransaksi;
import org.sadigit.entity.transaksi.Gangguan;
import org.sadigit.entity.transaksi.GangguanFoto;
import org.sadigit.entity.transaksi.GangguanKodeCTT;
import org.sadigit.entity.transaksi.Keluhan;
import org.sadigit.model.request.CreateGangguanRequest;
import org.sadigit.model.request.CreateGangguanV2Request;
import org.sadigit.model.request.CreateKeluhanRequest;
import org.sadigit.model.request.CreateKeluhanV2Request;
import org.sadigit.model.request.GetCancelRequest;
import org.sadigit.model.request.GetCttRequest;
import org.sadigit.model.request.GetDataGarduV2Request;
import org.sadigit.model.request.GetDataRequest;
import org.sadigit.model.request.GetRatingPlgnRequest;
import org.sadigit.model.response.CreateGangguanResponse;
import org.sadigit.model.response.DetailPengaduanFlowResponse;
import org.sadigit.model.response.DetailPengaduanResponse;
import org.sadigit.model.response.GarduNyalaResponse;
import org.sadigit.model.response.GarduNyalaV2Response;
import org.sadigit.model.response.GetCttResponse;
import org.sadigit.model.response.GetDataKode4Response;
import org.sadigit.model.response.GetDataKeteranganLokasi;
import org.sadigit.model.response.GetDataResponse;
import org.sadigit.model.response.GarduPadamResponse;
import org.sadigit.model.response.GarduPadamV2Response;
import org.sadigit.model.response.GarduRencanaResponse;
import org.sadigit.model.response.GarduRencanaV2Response;
import org.sadigit.model.response.GetFotoResponse;
import org.sadigit.model.response.GetHistoByIdpelResponse;
import org.sadigit.model.response.GetHistoriGangguanByIdResponse;
import org.sadigit.model.response.GetMasterJenisPengaduanResponse;
import org.sadigit.model.response.GetMasterJenisPengaduanResponseV2;
import org.sadigit.model.response.GetRatingPlnResponse;
import org.sadigit.model.response.base.ResponseModel;
import org.sadigit.model.response.base.ResponseModelGardu;
import org.sadigit.model.response.base.ResponseModelGarduV2;
import org.sadigit.model.response.base.ResponseModelOutReturn;
import org.sadigit.model.response.base.ResponseModelWithFlow;
import org.sadigit.repository.entity.saidisaifi.masterjaringan.MasterGarduDistribusiRepository;
import org.sadigit.repository.entity.transaksi.GangguanRepository;
import org.sadigit.repository.entity.transaksi.GangguanTMTransaksiRepository;
import org.sadigit.repository.entity.transaksi.KeluhanRepository;
import org.sadigit.repository.service.PlnMobileServiceRepository;
import io.quarkus.panache.common.Parameters;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.persistence.Tuple;
import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * @author Tias Mardiansyah
 */
@ApplicationScoped
@RequiredArgsConstructor
@Slf4j
public class PlnMobileService {

    private final GangguanRepository gangguanRepository;
    private final KeluhanRepository keluhanRepository;
    private final MasterGarduDistribusiRepository masterGarduDistribusiRepository;
    private final GangguanTMTransaksiRepository gangguanTMTransaksiRepository;
    private final PlnMobileServiceRepository plnMobileServiceRepository;

    public ResponseModel<GetHistoriGangguanByIdResponse> getHistoriGangguanById(String in_data) {

        ResponseModel<GetHistoriGangguanByIdResponse> response = new ResponseModel<>();
        switch (in_data.substring(0, 1)) {
            case "K" : {
                Keluhan keluhan = Keluhan.find("no_laporan", in_data).firstResult();
                response.setData(GetHistoriGangguanByIdResponse.fromKeluhan(keluhan));
            } break;
            case "G" : {
                Gangguan gangguan = Gangguan.find("no_laporan", in_data).firstResult();
                response.setData(GetHistoriGangguanByIdResponse.fromGangguan(gangguan));
            } break;
            default: throw new CustomException(null, ResponseModel.builder().message("DATA TIDAK DITEMUKAN").build());
        }
        
        if (response.getData() != null) {
            response.setMessage("DATA DITEMUKAN");
        } else {
            response.setMessage("DATA TIDAK DITEMUKAN");
        }

        return response;
    }

    public ResponseModelWithFlow<DetailPengaduanResponse, DetailPengaduanFlowResponse> getDetailPengaduan(String in_data) {
        String prefix = in_data.substring(0, 1);

        ResponseModelWithFlow<DetailPengaduanResponse, DetailPengaduanFlowResponse> response = new ResponseModelWithFlow<>();
        List<DetailPengaduanResponse> detailPengaduan = new ArrayList<>();
        List<DetailPengaduanFlowResponse> detailPengaduanFlow = new ArrayList<>();

        try {
            switch (prefix) {
                // keluhan
                case "K": {
                    response.setData(detailPengaduan);
                    response.setFlow(detailPengaduanFlow);
                    response.setMessage("Belum dapat mencari berdasarkan keluhan");
                }
                    break;
                // Gangguan
                case "G": {
                    log.info("MASUK");
                    List<Gangguan> gangguanList = Gangguan.find("no_laporan", in_data).list();

                    gangguanList.forEach(gangguan -> {
                        DetailPengaduanResponse detail = DetailPengaduanResponse.fromGangguanEntity(gangguan);
                        List<DetailPengaduanFlowResponse> flow = DetailPengaduanFlowResponse.fromGangguan(gangguan);
                        detailPengaduan.add(detail);
                        detailPengaduanFlow.addAll(flow);
                    });

                    response.setData(detailPengaduan);
                    response.setFlow(detailPengaduanFlow);
                }
                    break;
                default: {
                    // throw something for not founded code
                }
                    break;
            }

            if (detailPengaduan.isEmpty()) {
                response.setMessage("DATA TIDAK DITEMUKAN");
            } else {
                response.setMessage("DATA DITEMUKAN");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            throw new CustomException("TERJADI KESALAHAN SAAT MENGAMBIL DATA");
        }

        return response;
    }

    public ResponseModel<List<GetHistoByIdpelResponse>> getHistoriGangguanByIdpel(String in_data) {

        ResponseModel<List<GetHistoByIdpelResponse>> response = new ResponseModel<>();
        List<Tuple> histo = plnMobileServiceRepository.get10PengaduanTerbaruByIdPel(in_data);
        List<GetHistoByIdpelResponse> historiPengaduan = histo.stream().map(GetHistoByIdpelResponse::fromTuple).collect(Collectors.toList());

        response.setMessage("DATA DITEMUKAN");
        response.setData(historiPengaduan);
        return response;
    }

    public ResponseModel<List<GetRatingPlnResponse>> getRating(GetRatingPlgnRequest params) {
        List<RatingPln> ratingPln = RatingPln.find("noLaporan = ?1 AND nilai = ?2 AND keterangan = ?3",
                params.getP_reportnumber(), params.getP_nilai(), params.getP_keterangan()).list();
        List<GetRatingPlnResponse> ratingList = new ArrayList<>();

        ratingPln.forEach(rating -> {
            GetRatingPlnResponse ratingResponse = GetRatingPlnResponse.builder()
                    .no_laporan(rating.getNoLaporan())
                    .nilai(rating.getNilai())
                    .keterangan(rating.getKeterangan())
                    .createddate(rating.getCreateDate())
                    .build();

            ratingList.add(ratingResponse);
        });

        ResponseModel<List<GetRatingPlnResponse>> response = new ResponseModel<>();
        if (ratingList.isEmpty()) {
            response.setMessage("DATA TIDAK DITEMUKAN");
        } else {
            response.setMessage("DATA DITEMUKAN");
        }
        response.setData(new ArrayList<>());

        return response;
    }

    public CreateGangguanResponse createGangguan(CreateGangguanRequest params) {
        return gangguanRepository.create(params);
    }

    public CreateGangguanResponse createGangguanV2(CreateGangguanV2Request params) {
        return gangguanRepository.createV2(params);
    }

    public GetFotoResponse<List<Map<String, String>>> getFoto(String p_reportnumber) {

        List<GangguanFoto> gangguanFoto = GangguanFoto.find("gangguan.no_laporan", p_reportnumber).list();
        GetFotoResponse<List<Map<String, String>>> response = new GetFotoResponse<>();

        response.setOut_return(new ArrayList<>());
        gangguanFoto.forEach(foto -> {
            String urlFoto = "http://10.1.50.173:32172/yantek/" + foto.getFoto();
            response.getOut_return().add(Map.of("image.base64", urlFoto));
        });

        response.setOut_message("FOTO DITEMUKAN");
        return response;

    }

    public ResponseModel<List<GetCttResponse>> getCtt(GetCttRequest params) {

        StringBuilder sb = new StringBuilder();
        Parameters queryParams = new Parameters();

        sb.append("gangguan.no_laporan = :reportnumber ");
        queryParams.and("reportnumber", params.getP_reportnumber());

        sb.append("AND gangguan.id_pelanggan_no_meter = :idpel ");
        queryParams.and("idpel", Long.valueOf(params.getP_idpel()));

        sb.append("AND kode_41 = :kode41 ");
        queryParams.and("kode41", params.getP_kode41());

        sb.append("AND kode_44 = :kode44 ");
        queryParams.and("kode44", params.getP_kode44());

        sb.append("AND kode_37 = :kode37 ");
        queryParams.and("kode37", params.getP_kode37());

        sb.append("AND kode_47 = :kode47 ");
        queryParams.and("kode47", params.getP_kode47());

        if (params.getP_kode08() != null) {
            sb.append("AND kode_08 = :kode08 ");
            queryParams.and("kode08", params.getP_kode08());
        }

        if (params.getP_kode70() != null) {
            sb.append("AND kode_70 = :kode70 ");
            queryParams.and("kode70", params.getP_kode70());
        }

        if (params.getP_kode71() != null) {
            sb.append("AND kode_71 = :kode71 ");
            queryParams.and("kode71", params.getP_kode71());
        }

        if (params.getP_kode37() != null) {
            sb.append("AND kode_37 = :kode37 ");
            queryParams.and("kode37", params.getP_kode37());
        }

        if (params.getP_kode45() != null) {
            sb.append("AND kode_45 = :kode45 ");
            queryParams.and("kode45", params.getP_kode45());
        }

        // remove the last AND if it's present
        if (sb.toString().endsWith(" AND ")) {
            sb.setLength(sb.length() - 4);
        }

        List<GangguanKodeCTT> gangguanKodeCttList = GangguanKodeCTT.find(sb.toString(), queryParams).list();
        ResponseModel<List<GetCttResponse>> response = new ResponseModel<>();
        response.setData(new ArrayList<>());

        log.info("params: {}", params.toString());
        log.info("query: {}", sb.toString());
        log.info("gangguanKodeCttList: {}", gangguanKodeCttList.size());
        gangguanKodeCttList.forEach(ctt -> {
            var data = GetCttResponse.builder()
                    .id(ctt.getId())
                    .idpel(ctt.getGangguan().getId_pelanggan_no_meter().toString())
                    .reportnumber(ctt.getGangguan().getNo_laporan())
                    .tglcatat(ctt.getCreated_date() == null ? null : ctt.getCreated_date())
                    .createdate(ctt.getCreated_date() == null ? null
                            : ctt.getCreated_date().atZone(ZoneOffset.ofHours(7)).toInstant().toEpochMilli())
                    .kode08(ctt.getKode_08())
                    .kode70(ctt.getKode_70())
                    .kode71(ctt.getKode_71())
                    .kode41(ctt.getKode_41())
                    .kode44(ctt.getKode_44())
                    .kode45(ctt.getKode_45())
                    .kode46(ctt.getKode_46())
                    .kode37(ctt.getKode_37())
                    .kode47(ctt.getKode_47())
                    .kode08_result(ctt.getKode_08_hasil())
                    .kode70_result(ctt.getKode_70_hasil())
                    .kode71_result(ctt.getKode_71_hasil())
                    .kode41_result(ctt.getKode_41_hasil())
                    .kode44_result(ctt.getKode_44_hasil())
                    .kode45_result(ctt.getKode_45_hasil())
                    .kode46_result(ctt.getKode_46_hasil())
                    .kode37_result(ctt.getKode_37_hasil())
                    .kode47_result(ctt.getKode_47_hasil())
                    .build();

            response.getData().add(data);
        });

        response.setMessage("DATA DITEMUKAN");
        return response;
    }

    public ResponseModel<Object> getData(GetDataRequest params) {
        
        ResponseModel<Object> response = new ResponseModel<>();
        switch(params.getP_kode()) {
            case "1" : {
                try {

                    List<GetDataResponse> data = params.getP_input().equals("ALL") 
                        ? gangguanTMTransaksiRepository.getGangguanTMNotFlagged(null).stream().map(GetDataResponse::fromGangguanTM).collect(Collectors.toList())
                        : gangguanTMTransaksiRepository.getGangguanTMNotFlagged(Integer.valueOf(params.getP_input())).stream().map(GetDataResponse::fromGangguanTM).collect(Collectors.toList());

                    String message = Integer.valueOf(params.getP_input()) > 0
                        ? "BERHASIL BATAS = " + params.getP_input() 
                        : "BERHASIL";
                        
                    response.setData(data);
                    response.setMessage(message);
                } catch(NumberFormatException e) {
                    log.info("Error: {}", e.getMessage());
                    throw new CustomException(null, ResponseModel.builder().message("P_INPUT TIDAK SESUAI").build());
                }
                
            } break;
            case "2" : {
                GetDataResponse data = GetDataResponse.fromGangguanTM(gangguanTMTransaksiRepository.flagTransaksi(params.getP_input()));
                response.setData(data);
                response.setMessage("BERHASIL");
            } break;
            case "3" : {
                try {
                    List<GetDataResponse> data = params.getP_input().equals("ALL")
                        ? gangguanTMTransaksiRepository.getGangguanTMNotFlaggedNotSelesai(null).stream().map(GetDataResponse::fromGangguanTM).collect(Collectors.toList())
                        : gangguanTMTransaksiRepository.getGangguanTMNotFlaggedNotSelesai(Integer.valueOf(params.getP_input())).stream().map(GetDataResponse::fromGangguanTM).collect(Collectors.toList());

                    String message = Integer.valueOf(params.getP_input()) > 0
                        ? "BERHASIL BATAS = " + params.getP_input() 
                        : "BERHASIL";
                        
                    response.setData(data);
                    response.setMessage(message);
                } catch(Exception e) {
                    log.info("Error: {}", e.getMessage());
                }
            } break;
            case "4" : {
                Double latitude;
                Double longitude;
                
                try {
                    String[] koordinat = params.getP_input().split("#");
                    longitude = Double.parseDouble(koordinat[0]);
                    latitude = Double.parseDouble(koordinat[1]);
                } catch (NumberFormatException | NullPointerException e) {
                    throw new CustomException(null, ResponseModel.builder().message("KOORDINAT TIDAK VALID").build());
                }

                log.info("LAT: {}",latitude);
                log.info("LONG: {}", longitude);
                response.setData(masterGarduDistribusiRepository.findByGeomRadius(longitude, latitude).stream().map(GetDataKode4Response::fromGarduIndukDistribusi).collect(Collectors.toList()));
                response.setMessage("BERHASIL");
            } break;

            //what is this mess!!
            case "getMasterProvinsi" : {
                List<Provinsi> listProvinsi = Provinsi.findAll().list();
                response.setData(listProvinsi.stream().map(GetDataKeteranganLokasi::fromMasterProvinsi).collect(Collectors.toList()));
                response.setMessage("Data master");
            } break;
            case "getMasterKabupatenByKdProv" : {
                List<Provinsi> listProvinsi = Provinsi.find("id_provinsi", params.getP_input()).list();
                response.setData(listProvinsi.stream().map(GetDataKeteranganLokasi::fromMasterProvinsi).collect(Collectors.toList()));
                response.setMessage("Data master");
            } break;
            case "getMasterkecamatanByKdKab" : {
                List<Kecamatan> listKecamatan = Kecamatan.find("id_kota_kabupaten", params.getP_input()).list();
                response.setData(listKecamatan.stream().map(GetDataKeteranganLokasi::fromMasterKecamatan).collect(Collectors.toList()));
                response.setMessage("Data master"); 
            } break;
            case "getMasterDesaByKdKec" : {
                List<MasterKelurahan> data = MasterKelurahan.find("id_kecamatan", params.getP_input()).list();
                response.setData(data.stream().map(GetDataKeteranganLokasi::fromMasterKelurahan).collect(Collectors.toList()));
                response.setMessage("Data master");
            } break;
            case "getMasterPemda" : {
                // response.setData(Unit.find())
                response.setMessage("Data master");
            } break;
            case "getSubKeluhanEV" : {
                response.setData(SubTipePermasalahan.find("tipeKeluhan.issuetypeid", 52).list());
                response.setMessage("Data master");
            } break;
            default: throw new CustomException(null, ResponseModel.builder().message("KODE TIDAK DITEMUKAN").build());
        }
        
        return response;
    }

    public ResponseModel<Map<String, String>> createKeluhan(CreateKeluhanRequest params) {
        return keluhanRepository.create(params);
    }

    public ResponseModel<Map<String, String>> createKeluhanV2(CreateKeluhanV2Request params) {
        return keluhanRepository.createV2(params);
    }

    public ResponseModel<List<GetMasterJenisPengaduanResponse>> getMasterJenisPengaduan() {

        List<GetMasterJenisPengaduanResponse> jenisPengaduan = new ArrayList<>();
        ResponseModel<List<GetMasterJenisPengaduanResponse>> response = new ResponseModel<>();

        List<TipeKeluhan> tipeKeluhan = TipeKeluhan.find("projectid in (1,2) and issuetypeid between 1 and 32").list();
        tipeKeluhan.forEach(tipe -> {
            List<GetMasterJenisPengaduanResponse> subTipeKeluhan = tipe.getSubTipeKeluhan().stream().map(GetMasterJenisPengaduanResponse::fromSubTipeKeluhan).collect(Collectors.toList());
            jenisPengaduan.addAll(subTipeKeluhan);
        });

        response.setMessage("BERHASIL");
        response.setData(jenisPengaduan);
        return response;
    }

    public ResponseModel<List<GetMasterJenisPengaduanResponseV2>> getMasterJenisPengaduanV2() {

        List<GetMasterJenisPengaduanResponseV2> jenisPengaduan = new ArrayList<>();
        ResponseModel<List<GetMasterJenisPengaduanResponseV2>> response = new ResponseModel<>();

        List<TipeKeluhan> tipeKeluhan = TipeKeluhan.find("projectid = 1 and isactive = 1").list();
        tipeKeluhan.forEach(tipe -> {
            List<GetMasterJenisPengaduanResponseV2> subTipeKeluhan = tipe.getSubTipeKeluhan().stream().map(GetMasterJenisPengaduanResponseV2::fromSubTipeKeluhan).collect(Collectors.toList());
            jenisPengaduan.addAll(subTipeKeluhan);
        });

        response.setMessage("BERHASIL");
        response.setData(jenisPengaduan);
        return response;
    }

    public ResponseModelOutReturn<String> batalkanPengaduan(GetCancelRequest params) {
        try {
            Gangguan gangguan = Gangguan.find("no_laporan = ?1 and id_pelanggan_no_meter = ?2", params.getP_reportnumber(), params.getP_userid()).firstResult();
            gangguan.setStatus_akhir("batal");
            gangguan.getGangguan_transaksi().forEach(transaksi -> transaksi.setJenis("batal"));
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
            throw new CustomException("GAGAL MEMBATALKAN PENGADUAN");
        }

        ResponseModelOutReturn<String> response = new ResponseModelOutReturn<>();
        response.setOutreturn("true");
        response.setOutmessage("Berhasil Dilakukan pembatalan");
        return response;
    }

    @Transactional
    public ResponseModelGardu<GarduPadamResponse, GarduNyalaResponse, GarduPadamResponse, GarduRencanaResponse> getDataGardu() {

        List<MasterGarduIndukTransaksi> garduIndukPadamAllList = MasterGarduIndukTransaksi.findAll().list();

        List<GarduPadamResponse> garduPadamList = new ArrayList<>();
        List<GarduNyalaResponse> garduNyalaList = new ArrayList<>();
        List<GarduRencanaResponse> garduRencanaList = new ArrayList<>();
        List<GarduPadamResponse> garduPadamNewList = new ArrayList<>();

        List<String> readedGarduId = new ArrayList<>();

        garduIndukPadamAllList.forEach(gardu -> {
            switch(gardu.getStatus()) {
                case "padam": {
                    if (gardu.getIs_read() == false) {
                        garduPadamNewList.add(GarduPadamResponse.fromMasterGarduIndukTransaksi(gardu));
                        readedGarduId.add(gardu.getId().toString());
                    } else {
                        garduPadamList.add(GarduPadamResponse.fromMasterGarduIndukTransaksi(gardu));
                    }
                } break;
                case "nyala": {
                    garduNyalaList.add(GarduNyalaResponse.fromMasterGarduIndukTransaksi(gardu));
                } break;
                case "rencana": {
                    garduRencanaList.add(GarduRencanaResponse.fromMasterGarduIndukTransaksi(gardu));
                } break;
            }
        });

        readedGarduId.forEach( d -> {
            log.info("readed gardu id: {}", d);
        });

        if (readedGarduId.size() > 0) {
            log.info("readed gardu id: {}", readedGarduId.toArray());
            MasterGarduIndukTransaksi.update("is_read = true where id in ?1", readedGarduId.toArray());
        }
        
        ResponseModelGardu<GarduPadamResponse, GarduNyalaResponse, GarduPadamResponse, GarduRencanaResponse> response = new ResponseModelGardu<>();
        response.setDesc("BERHASIL");
        response.setOut_data(garduPadamList);
        response.setOut_data_padam(garduPadamNewList);
        response.setOut_data_nyala(garduNyalaList);
        response.setOut_data_rencana(garduRencanaList);
        
        return response;
    }

    @Transactional
    public ResponseModelGarduV2<Object> getDataGarduV2(GetDataGarduV2Request params) {

        ResponseModelGarduV2<Object> response = new ResponseModelGarduV2<>();

        switch (params.getP_kode()) {
            //melihat jumlah event tersedia
            case 1: {    
                Long eventCount = 0L;
                switch (params.getP_jenis()) {
                    case 1: eventCount = MasterGarduIndukTransaksi.count("status", "PADAM"); break;
                    case 2: eventCount = MasterGarduIndukTransaksi.count("status", "NYALA");  break;
                    case 3: eventCount = MasterGarduIndukTransaksi.count("status", "PERENCANAAN");  break;
                    default : throw new GetDataGarduV2Exception("99","GAGAL !! p_jenis HARUS 1/2/3");
                }

                response.setP_return(Map.of("jumlah", eventCount));
            } break;

            // untuk mengambil data event tersedia
            case 2: {
                switch (params.getP_jenis()) {
                    case 1: {
                        List<MasterGarduIndukTransaksi> gardu = (params.getP_jumlah() != null && params.getP_jumlah() > 0) 
                            ? MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "PADAM").page(0, params.getP_jumlah()).list()
                            : MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "PADAM").list();

                        var garduPadam = gardu.stream().map(GarduPadamV2Response::fromMasterGarduIndukTransaksi).collect(Collectors.toList());
                        response.setP_return(garduPadam);
                    }; break;
                    case 2: {
                        List<MasterGarduIndukTransaksi> gardu = (params.getP_jumlah() != null && params.getP_jumlah() > 0) 
                            ? MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "NYALA").page(0, params.getP_jumlah()).list()
                            : MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "NYALA").list();

                        List<GarduNyalaV2Response> garduPadam = gardu.stream().map(GarduNyalaV2Response::fromMasterGarduIndukTransaksi).collect(Collectors.toList());
                        response.setP_return(garduPadam);
                    }; break;
                    case 3: {   
                        List<MasterGarduIndukTransaksi> gardu = (params.getP_jumlah() != null && params.getP_jumlah() > 0) 
                            ? MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "PERENCANAAN").page(0, params.getP_jumlah()).list()
                            : MasterGarduIndukTransaksi.find("status = ?1 and (is_read = false or is_read is NULL)", "PERENCANAAN").list();

                        List<GarduRencanaV2Response> garduPadam = gardu.stream().map(GarduRencanaV2Response::fromMasterGarduIndukTransaksi).collect(Collectors.toList());
                        response.setP_return(garduPadam);
                    }; break;
                    default : throw new GetDataGarduV2Exception("99","GAGAL !! p_jenis HARUS 1/2/3");
                }
            } break;

            // flagging data yang telah terambil
            case 3: {
                try {
                    if (params.getP_nolaporan() == null || params.getP_nolaporan().isEmpty()) {
                        throw new GetDataGarduV2Exception("99","NOMOR LAPORAN TIDAK BOLEH KOSONG");
                    }
    
                    if (params.getP_kodegardu() == null || params.getP_kodegardu().isEmpty()) {
                        throw new GetDataGarduV2Exception("99","KODE GARDU TIDAK BOLEH KOSONG");
                    }
          
                    String status = "";
                    switch (params.getP_jenis()) {
                        case 1: status = "PADAM"; break;
                        case 2: status = "NYALA"; break;
                        case 3: status = "PERENCANAAN"; break;
                        default : throw new GetDataGarduV2Exception("99","GAGAL !! p_jenis HARUS 1/2/3");
                    }

                    int updatedRow = MasterGarduIndukTransaksi.update("is_read = true where status = ?1 and no_laporan = ?2 and garduInduk.kode = ?3", status, params.getP_nolaporan(), params.getP_kodegardu());
                    log.info("Updated: {}", updatedRow);

                    response.setP_return(Map.of("info", "Berhasil Flagging, no_laporan:" + params.getP_nolaporan() + ", kode_gardu:" + params.getP_kodegardu()));
                } catch (HibernateException e) {
                    throw new GetDataGarduV2Exception("99","GAGAL !! "+e.getMessage());
                } 
            } break;
            default: throw new GetDataGarduV2Exception("99","GAGAL !! p_kode HARUS 1/2/3");
        }

        return response;
    }

}
